//-----------------------------------------------------------------------------
// GA2K
//
// ISDL, Doshisha University Distributed Genetic Algorithm Software
// Version 1.2
//-----------------------------------------------------------------------------
// Parallel Genetic Algorithm Team of Intelligent Systems Design Laborato
// ry, Doshisha University.
//
// Version 0.1: 1999/05/05
// Copyright (C) 1999 Jun-ichi Yoshida and Ikki Ohmukai, All rights reserved.
// Version 1.0: 2001/12/13
// Copyright (C) 2001 Jun-ichi Yoshida, Tomoyuki Hiroyasu and Mitsunori Miki,
// All rights reserved.
// Version 1.1: 2002/01/17
// Copyright (C) 2002 Jun-ichi Yoshida, Masaki Sano, Tomoyuki Hiroyasu and Mitsunori Miki, All rights reserved.
// Version 1.2: 2002/06/04
// Copyright (C) 2002 Jun-ichi Yoshida, Masaki Sano, Tomoyuki Hiroyasu and Mitsunori Miki, All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notce and this list of conditions.
// 2. Redistributions in binary form must reproduce the above  copyright
//    notice and this list of conditions in the documentation and/or other
//    materials provided with the distribution.
// 3. The names of copyrighters of this software  should not be used to
//    endorse or promote products derived from this software without specific
//    prior written permission.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// ga2k.h
//-----------------------------------------------------------------------------

#ifndef _GA2K_H_
#define _GA2K_H_


#include <vector>
#include <fstream>
using namespace std;

#include "mtrand.h"
#include "randring.h"

#define INITIAL_POPULATION_FILE "design_variable.ini"


class Gene
{
  public:
	//data
	vector<char>	bitString;
	mutable double	fitness;

	//operator
	bool operator<(const Gene& gene) const;

	//method (constructor)
	Gene();
	~Gene();
	void initialize(int geneLength);
	void initialize(int geneLength, char value);
	void initialize(int geneLength, const vector<char>& code);

	//method (operator)
	void visualize();
};

class Island
{
  public:
	//data
	int		islandId;
	int		numIsland;

	int		populationSize;

	double	crossoverRate;
	double	mutationRate;

	int		numElite;

	double	migrationRate;
	int		migrantSize;
	int		nativeSize;
	int 	tournamentSize;

	int		geneLength;
	int		variableLength;
	int		numVariable;
	double	openMax;
	double	closedMin;
	double	delta;
	double	finValue;

	int		functionType;
	int		typeOfEncode;
	int		numCrossoverPoint;
	int		selectionType;

	mutable int		numEvaluate;

	double	bestFitness;

	vector<double>	roulette;
	vector<int>		crossoverTable;

//gene
	vector<Gene>	gene;
	vector<Gene>	migrant;
	vector<Gene>	elite;
	vector<Gene>	newGeneration;

//method (constructor)
	Island();
	~Island();
	void initialize(int id, int numEnv, std::ifstream& fin);

//method (operator)
	void immigrate();
	void crossover();
	void nPointCrossover();
	void mutate();
	void evaluate();
	void reproduce();
	void rouletteSelection();
	void tournamentSelection();
	void elitePreservation();
	void setGeneFromFile(std::ifstream& fin);


	void fitness(const Gene& gene) const;
};

class Globe
{
  public:
	//data
	vector<Island>	island;
	Island			island_0;

	int					generation;
	int					maxGeneration;
	int					convGeneration;

	int					migrationInterval;
	int					numIsland;
	int					populationSize;
	int					totalPopulationSize;
	int					migrantSize;
	int					nativeSize;

	int 				fInitialPopulation;

	double				maxFitness;
	int					maxTrial;
	double				sumMaxFitness;
	int					sumGeneration;
	int					numConvergent;
	int					totalEvaluate;
	int					sumTotalEvaluate;

	vector<double>		sumFitness;

	Gene				elite;
	bool				fStop;
	bool				fConv;
	bool				fTotalPopSize;
	bool				fOutput;

	//method (constructor)
	Globe();
	~Globe();
	void initialize();
	void initialize(int argc, char *argv[]);
	void establish();

	//method (operator)
	void emigrate();
	void execute();
	void showParameters();
	void finalize(bool b);
};


#endif //_GA2K_H_
